/** @file rob_rel_obs.cpp

Tutorial, supervisory control

@ingroup Tutorials 

@include rob_rel_obs.cpp

*/

#include "libfaudes.h"

// make the faudes namespace available to our program
using namespace faudes;

// main program
int main() {

    std::cout << "Reading generators..." << std::endl;
    // read generators from files
    Generator g;
    g.Read("data/rel_obs_g.gen");
    Generator a;
    a.Read("data/rel_obs_a.gen");
    Generator h;
    h.Read("data/rel_obs_h.gen");

    std::cout << "Reading event sets..." << std::endl;
    // read the set of observable events from file
    EventSet obsEvents;
    obsEvents.Read("data/rel_obs_obs_events.alph");


    std::cout << "Input generators are read form directory \"data\". Directory also contains theirs visual representation." << std::endl << std::endl;

    // now we test if K is realtively observable with respect to prefix-closure(C),
    // G and Po.
    // Po is natural projection: E* -> Eo. Eo = obsEvents - set of observable events
    // Lm(A) = C, Lm(H) = K

    std::cout << "Lm(A) = C, Lm(H) = K, Eo = {b, a, o}" << std::endl;
    std::cout << "Testing if language Lm(H) = K is relatively observable with respect to prefix-closure(C), G and Po with result:" << std::endl;
    if (isRelativelyObservable(g, a, h, obsEvents)) {
        std::cout << "Language Lm(H) = K is relatively observable with respect to prefix-closure(C), "
                     "G and Po!" << std::endl;
    } else {
        std::cout << "Language Lm(H) = K is not relatively observable with respect to prefix-closure(C), "
                     "G and Po!" << std::endl;
    }

    std::cout << std::endl << "Computing the supremal relatively observable sublanguage of K with respect to prefix-closure(C), "
                              "G and Po and creating visual representation in directory \"results\" named \"rel_obs_supK.png\"..." << std::endl << std::endl;

    // now we compute the supremal relatively observable sublanguage with respect to prefix-closure(C),
    // G and Po; funtcion returns it as output generator parameter
    // inputs are the same as in example above

    Generator supRelObsLan;
    supRelativelyObservable(g, a, h, supRelObsLan, obsEvents);

    std::cout << "Testing if computed language is relatively observable with respect to prefix-closure(C), G and Po with result:" << std::endl;
    supRelObsLan.GraphWrite("results/rel_obs_supK.png");

    // now we can further work with generator representing the supremal language, e.g. test it again

    if (isRelativelyObservable(g, a, supRelObsLan, obsEvents)) {
        std::cout << "Computed language is relatively observable with respect to prefix-closure(C), "
                     "G and Po!" << std::endl;
    } else {
        std::cout << "Computed language is not relatively observable with respect to prefix-closure(C), "
                     "G and Po!" << std::endl;
    }

    // done
    return 0;
}



